package com.zcxk.dao;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zcxk.dto.TenantDto;
import com.zcxk.entity.TenantEntity;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @description
 * @auto wangli
 * @data 2020-10-26 10:05
 */
@Mapper
public interface TenantMapper  {

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    TenantDto selectById(Integer id);
    /**
     * 通过租户标识查询单条数据
     *
     * @param code 主键
     * @return 实例对象
     */
    TenantDto selectByCode(String code);
    /**
     * 查询全部
     *
     * @return 对象列表
     */
    List<TenantDto> selectAll();

    /**
     * 通过实体作为筛选条件查询
     *
     * @param tenantDto 实例对象
     * @return 对象列表
     */
    List<TenantDto> selectList(TenantDto tenantDto);

    /**
     * 新增数据
     *
     * @param tenantEntity 实例对象
     * @return 影响行数
     */
    int insert(TenantEntity tenantEntity);

    /**
     * 批量新增
     *
     * @param tenantEntities 实例对象的集合
     * @return 影响行数
     */
    int batchInsert(List<TenantEntity> tenantEntities);

    /**
     * 修改数据
     *
     * @param tenantEntity 实例对象
     * @return 影响行数
     */
    int update(TenantEntity tenantEntity);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 影响行数
     */
    int deleteById(Integer id);

    int deleteUserByTenantId(Integer id);

    /**
     * 查询总数据数
     *
     * @return 数据总数
     */
    int count();

    /**
     * 根据租户名字查询租户id
     * @param tenantName
     * @return
     */
    List<Integer> countIds( String tenantName);
    /**
     *  分页查询
     * @param page
     * @param tenantDto
     * @return
     */
    IPage<TenantDto> selectPage( IPage<TenantDto> page, TenantDto tenantDto);

    /**
     * 创建租户菜单
     * @return
     */
    int createTenantMenu( @Param("userName") String userName,@Param("tenantId") String tenantId, @Param("menuIds") List<Integer> menuIds);

    /**
     * 通过租户code查询租户菜单id
     * @param tenantId
     * @return
     */
    List<Integer> selectTenantMenuIds(String tenantId);

    /**
     * 删除租户菜单
     * @param tenantId
     * @param menuIds
     * @return
     */
    Integer deleteTenantMenuIds( @Param("tenantId") String tenantId,@Param("menuIds")List<Integer> menuIds);
}
